﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using AIOPC104Net;  // The namespace exposes the AIOPC104 Class interface (AIO167.dll)



namespace Sample0
{
    public partial class Form1 : Form
    {
        public AIO16A cAIO16A = new AIO16A();
        
        // Card info:
        public int channels = 16;
        public double dac0mult, dac1mult;
        
        public const int ERROR_SUCCESS = 0;
        
        // Num channels we could be using:
        public const int numAnalogInputs = 16;

        // Array of controls for GUI updates:
        public TextBox[] ChannelVolts = new TextBox[numAnalogInputs]; // buffer data for display


        // Data used to discover cards in the system:
        UInt32 Status = 0;
        const UInt32 MAX_CARDS = 10;
        public Int32 NumCards = 0;

        // This is the basic data that we will be using to interface with the card:
        public Int32 CardNum = 0;   // The index of the one card we will be using 0
        public UInt32 Offset = 0;   // This the offset that will be used based on the base address of the card

        //public Int32 UseFIFO = 0;
            
        bool RunFlag;
        public bool TimerEnabled = false;
      

        public int i = 0;

        public struct TCardData 
	    {
		    public bool IsValid;
            public bool IsSelected;
            public UInt32 DeviceID;
            public UInt32 Base;     // This the result of findcards and the base address of the card
	    };
        public TCardData CardData; // only one struct
        

        // Interval Timer fou GUI update:
        // (Note: A Windows Forms timer designed for single threaded environments not a System Timer 55 ms min res)
        static System.Windows.Forms.Timer myTimer = new System.Windows.Forms.Timer();


        public Form1()
        {
            InitializeComponent();

            // Initialize default values:

            // Arrays of the controls:
            ChannelVolts[0] = textBoxVolts0;
            ChannelVolts[1] = textBoxVolts1;
            ChannelVolts[2] = textBoxVolts2;
            ChannelVolts[3] = textBoxVolts3;
            ChannelVolts[4] = textBoxVolts4;
            ChannelVolts[5] = textBoxVolts5;
            ChannelVolts[6] = textBoxVolts6;
            ChannelVolts[7] = textBoxVolts7;
            ChannelVolts[8] = textBoxVolts8;
            ChannelVolts[9] = textBoxVolts9;
            ChannelVolts[10] = textBoxVolts10;
            ChannelVolts[11] = textBoxVolts11;
            ChannelVolts[12] = textBoxVolts12;
            ChannelVolts[13] = textBoxVolts13;
            ChannelVolts[14] = textBoxVolts14;
            ChannelVolts[15] = textBoxVolts15;


            // Check device info:
            UInt32 iNumCards = AIOPC104.GetNumCards();

	        // Detect correct card is present:
            UInt32 i, NumCards, Status ;//Buf;
            UInt32 retval = 0; 

	        CardNum = 0xFFFF;
            NumCards = AIOPC104.GetNumCards();
	        for( i = 0;  i < NumCards; i++ )
	        {
                Status = AIOPC104.QueryCardID(i, out retval);
		        if( Status != ERROR_SUCCESS ) continue; //Failed to query ID.
		        if( retval != 0x1ECE8) continue; //Not the ID of the 104-AIO16E.
                Status = AIOPC104.QueryCardBase(i, out retval);
		        if( Status != ERROR_SUCCESS) continue;

		        //We've successfully queried it and found the card we're looking for.
		        CardNum = (int)i;
		        break;
	        }


	        if ( CardNum == 0xFFFF) 
	        {
                String strtemp;
                strtemp = " No 104-AIO16E found; make sure it's installed in Device Manager.\r\n ";
                textBoxStatus.Text = strtemp;
	        }
	        else
	        {
		        String Buff;
                Buff = " Card at index: ";
                Buff += Convert.ToString(CardNum);
                Buff += " 104-AIO16E card found.\r\n Click Begin to begin acquisition.\r\n ";
                textBoxStatus.Text = Buff;
	        }

            textBoxStatus.Text += " This sample demonstrates basic operation of the board.\r\n ";
            
            // Add the event and the event handler for the method that will 
            // process the timer event to the timer:
            myTimer.Tick += new EventHandler(TimerEventProcessor);

            // Set the timer interval in miliseconds and start 55ms min resolution:
            myTimer.Interval = 200;
        }
        
        private void Form1_Load(object sender, EventArgs e)
        {

        }
       
        private void TimerEventProcessor(Object myObject, EventArgs myEventArgs)
        {
            // This is the method that runs when the timer event is raised:
            myTimer.Stop();

            UpdateGUIState();

            myTimer.Enabled = true;
        }

        private void UpdateGUIState()
        {
            // Get data and display:

            // Stop timer:
            TimerEnabled = false;
            myTimer.Stop();
	        
            // Sample data:
	        double bpAdjust, gainAdjust, value;

            //determine the range and if the card is unipolar or bipolar
	        // this is used to convert the counts to volts
	        if( ((cAIO16A.jumpers) & 0x1) != 0 )
		        bpAdjust = 0.5;
	        else
		        bpAdjust = 0.0;

	        if ( ((cAIO16A.jumpers) & 0x4) != 0 )
		        gainAdjust = 10.0;
	        else
		        gainAdjust = 20.0;

	        AIOPC104.RelOutPortB(CardNum, 1, 0);  //issue a software start (in this case start SCAN)

	        //read the values from the FIFO, convert to volts, and display to the user
            // raw data dunp we should format this to 4 decimal places
	        value = ((AIOPC104.RelInPort(CardNum, 0)/65536.0) - bpAdjust) * gainAdjust;
	        ChannelVolts[0].Text = value.ToString("F4");

	        value = ((AIOPC104.RelInPort(CardNum, 0)/65536.0) - bpAdjust) * gainAdjust;
            ChannelVolts[1].Text = value.ToString("F4");

	        value = ((AIOPC104.RelInPort(CardNum, 0)/65536.0) - bpAdjust) * gainAdjust;
            ChannelVolts[2].Text = value.ToString("F4");

	        value = ((AIOPC104.RelInPort(CardNum, 0)/65536.0) - bpAdjust) * gainAdjust;
            ChannelVolts[3].Text = value.ToString("F4");

	        value = ((AIOPC104.RelInPort(CardNum, 0)/65536.0) - bpAdjust) * gainAdjust;
            ChannelVolts[4].Text = value.ToString("F4");

	        value = ((AIOPC104.RelInPort(CardNum, 0)/65536.0) - bpAdjust) * gainAdjust;
            ChannelVolts[5].Text = value.ToString("F4");

	        value = ((AIOPC104.RelInPort(CardNum, 0)/65536.0) - bpAdjust) * gainAdjust;
            ChannelVolts[6].Text = value.ToString("F4");

	        value = ((AIOPC104.RelInPort(CardNum, 0)/65536.0) - bpAdjust) * gainAdjust;
            ChannelVolts[7].Text = value.ToString("F4");

	        if ( channels == 16 )
	        {
		        value = ((AIOPC104.RelInPort(CardNum, 0)/65536.0) - bpAdjust) * gainAdjust;
                ChannelVolts[8].Text = value.ToString("F4");

		        value = ((AIOPC104.RelInPort(CardNum, 0)/65536.0) - bpAdjust) * gainAdjust;
                ChannelVolts[9].Text = value.ToString("F4");

		        value = ((AIOPC104.RelInPort(CardNum, 0)/65536.0) - bpAdjust) * gainAdjust;
                ChannelVolts[10].Text = value.ToString("F4");

		        value = ((AIOPC104.RelInPort(CardNum, 0)/65536.0) - bpAdjust) * gainAdjust;
                ChannelVolts[11].Text = value.ToString("F4");

		        value = ((AIOPC104.RelInPort(CardNum, 0)/65536.0) - bpAdjust) * gainAdjust;
                ChannelVolts[12].Text = value.ToString("F4");

		        value = ((AIOPC104.RelInPort(CardNum, 0)/65536.0) - bpAdjust) * gainAdjust;
                ChannelVolts[13].Text = value.ToString("F4");

		        value = ((AIOPC104.RelInPort(CardNum, 0)/65536.0) - bpAdjust) * gainAdjust;
                ChannelVolts[14].Text = value.ToString("F4");

		        value = ((AIOPC104.RelInPort(CardNum, 0)/65536.0) - bpAdjust) * gainAdjust;
                ChannelVolts[15].Text = value.ToString("F4");
	        }
	        //output the dac values and then put outputted values to screen
	        if( (cAIO16A.dacArange & 0x1) != 0 )
                value = cAIO16A.DAC(0, dac0mult * 0.5);
	        else
                value = cAIO16A.DAC(0, dac0mult);

            numericUpDownDAC0.Value = (decimal)value;

            if ((cAIO16A.dacBrange & 0x1) != 0)
                value = cAIO16A.DAC(1, dac1mult * 0.5);
	        else
                value = cAIO16A.DAC(1, dac1mult);
            
            numericUpDownDAC1.Value = (decimal)value;

            // Start timer:
            TimerEnabled = true;
            myTimer.Enabled = true;
         }
    
        private void btnPerformIO_Click(object sender, EventArgs e)
        {
            if (TimerEnabled)
	        {
		        // Stop timer:
                TimerEnabled = false;
                myTimer.Stop();
                btnPerformIO.Text = "Perform I/O";
	        }
	        else
	        {
                // Check for card and state then configure it then start a timer
                if (CardNum == 0xFFFF)
                {
                    Console.Beep(1000, 100);
                    String strtemp = "No 104-AIO16E detected; make sure it's installed in Device Manager.\r\n ";
                    textBoxStatus.Text = strtemp;
                    return;
                }

                AIOPC104.RelOutPortB(CardNum, 0x1B, 0x1F); // reset all possible states on the device, most desirably the ADC FIFO -- note: the value $10 should accomplish this as well.

                //Initialize the global variables and then calibrate the card
                cAIO16A.init();
                cAIO16A.doCAL();

                //determine if card is in single ended or differential mode
                if ((cAIO16A.jumpers & 0x2) != 0)
                    channels = 16;
                else
                    channels = 8;

                //tell the card to take one reading per channel using Scan Start
                AIOPC104.RelOutPortB(CardNum, 0x11, 0x04); //Set ADC SCAN MODE and SOFTWARE START
                //tell the board which channels we are interested in. bits 0-3 are the start
                // channel and bits 4-7 are the end channel
                AIOPC104.RelOutPortB(CardNum, 0x06, (byte)((channels - 1) * 0x10)); //setup channels zero to maxchan

                //determine the dac range and output it to the user
                String strTemp;
                if ((cAIO16A.dacArange & 0x1) == 1)
                {
                    strTemp = "DAC 0 output range is 0 to 5 volts";
                    labelDAC0Range.Text = strTemp;
                }
                else
                {
                    strTemp = "DAC 0 output range is 0 to 10 volts";
                    labelDAC0Range.Text = strTemp;
                }

                if ((cAIO16A.dacBrange & 0x1) == 1)
                {
                    strTemp = "DAC 1 output range is 0 to 5 volts";
                    labelDAC1Range.Text = strTemp;
                }
                else
                {
                    strTemp = "DAC 1 output range is 0 to 10 volts";
                    labelDAC1Range.Text = strTemp;
                }

                // Start timer:
                TimerEnabled = true;
                myTimer.Enabled = true;
                btnPerformIO.Text = "Stop I/O";
	        }

        }
        
        private void btnExit_Click(object sender, EventArgs e)
        {
            // Right now this is only called on button click not window close ALtF4 etc:

            // Kill the timer:
            myTimer.Stop();
            myTimer.Enabled = false;
            myTimer.Dispose();

            // Close the form and Application:
            this.Close();
        }

        private void numericUpDownDAC0_ValueChanged(object sender, EventArgs e)
        {
            // The control will clip the values between 0-10
            // The timer loop will modify the values so we have to control the user inputs inc/dec from the spinner arrows:
            if (TimerEnabled)
            {
                // Stop timer:
                TimerEnabled = false;
                myTimer.Stop();

                // Adjust:
                if (((double)numericUpDownDAC0.Value - dac0mult) > .1)
                    dac0mult += 1.0;
                else if (((double)numericUpDownDAC0.Value - dac0mult) < -.1)
                    dac0mult -= 1.0;

                // Start timer:
                TimerEnabled = true;
                myTimer.Enabled = true;
            }

        }

        private void numericUpDownDAC1_ValueChanged(object sender, EventArgs e)
        {
            if (TimerEnabled)
            {
                // Stop timer:
                TimerEnabled = false;
                myTimer.Stop();

                // Adjust:
                if (((double)numericUpDownDAC1.Value - dac1mult) > .1)
                    dac1mult += 1.0;
                else if (((double)numericUpDownDAC1.Value - dac1mult) < -.1)
                    dac1mult -= 1.0;

                // Start timer:
                TimerEnabled = true;
                myTimer.Enabled = true;
            }
        }


    }
}
